# jan.kazil@noaa.gov 2024-08-23 20:32:56 -06:00

import numpy as np
import matplotlib.pyplot as plt
import os

#
# Function to read a Fortran binary file
#

def read_fortran_binary(filename):
    with open(filename, 'rb') as file:
        # Read dimensions
        x_n = np.fromfile(file, dtype=np.int32, count=1)[0]
        y_n = np.fromfile(file, dtype=np.int32, count=1)[0]
        z_n = np.fromfile(file, dtype=np.int32, count=1)[0]
        
        # Read arrays
        x = np.fromfile(file, dtype=np.float64, count=x_n)
        y = np.fromfile(file, dtype=np.float64, count=y_n)
        z = np.fromfile(file, dtype=np.float64, count=z_n)
        
        # Read the qv_xyz 3D array
        qv_xyz = np.fromfile(file, dtype=np.float64, count=x_n * y_n * z_n)
        qv_xyz = qv_xyz.reshape((x_n, y_n, z_n), order = 'F')
    
    return x_n, y_n, z_n, x, y, z, qv_xyz

#
# Function to plot a 2D slice at a given z-level
#

def plot_2d_slice_at_z(x, y, qv_xyz, z_index, z, plot_file):
    plt.figure(figsize=(10, 8))
    plt.contourf(x, y, qv_xyz[:, :, z_index].T, cmap='viridis')
    colorbar = plt.colorbar(label='Water vapor (kg kg-1)')
    plt.xlabel('x (m)')
    plt.ylabel('y (m)')
    plt.title(f'Water vapor at level {z_index}, z = {z} m')
    plt.savefig(plot_file)
    plt.close()

#
# Function that calculates the mean profile
#

def plot_vertical_profile_of_mean(qv_xyz, z,plot_name):
    # Compute the horizontal mean (average over x and y dimensions) for each z level
    mean_profile = np.mean(qv_xyz, axis=(0, 1))

    # Plot the vertical profile
    plt.figure(figsize=(6, 8))
    plt.plot(mean_profile, z, marker='o')
    plt.xlabel('kg kg-1')
    plt.ylabel('z')
    plt.title('Vertical profile of horizontal mean of water vapor')
    plt.grid(True)
    #plt.show()
    plt.savefig(plot_name)
    plt.close()

#
# Plot mean profiles
#

for filename in ['qv_without_anomaly.bin','qv_with_anomaly.bin']:
  
  plot_dir = 'plots'
  
  os.makedirs(plot_dir, exist_ok=True)
  
  x_n, y_n, z_n, x, y, z, qv_xyz = read_fortran_binary(filename)
  
  # Plot the vertical profile of the horizontal mean
  plot_file = os.path.join(plot_dir,filename + '.mean_profile.png')
  plot_vertical_profile_of_mean(qv_xyz, z, plot_file)

#
# Plot 2D slices at every level
#

for filename in ['qv_with_anomaly.bin']:
  
  plot_dir = 'plots'
  
  os.makedirs(plot_dir, exist_ok=True)
  
  x_n, y_n, z_n, x, y, z, qv_xyz = read_fortran_binary(filename)
  
  for z_index in range(1,z_n):
    plot_file = os.path.join(plot_dir,filename + '_level=' + f'{z_index:05}' + '.png')
    plot_2d_slice_at_z(x, y, qv_xyz, z_index, z[z_index],plot_file)

