! jan.kazil@noaa.gov 2024-08-23 20:32:56 -06:00

MODULE module_water_vapor_initialization
  
  !
  ! Module with routines to initialize water vapor with a mesoscale anomaly.
  !
  
  ! - initialize_water_vapor_mesoscale_anomaly
  !
  !   To be used for initialization in CP-MIP simulations. May require modification when MPI is used.
  !
  ! - initialize_water_vapor_z and initialize_water_vapor_xyz
  !
  !   Only needed for TEST_RUN.
  
CONTAINS
  
  SUBROUTINE initialize_water_vapor_mesoscale_anomaly(x_n,y_n,z_n,dx,dy,z,qv_xyz, &
    center_x,center_y,base,top,radius,dq)
    
    ! Takes a 3D water vapor array qv_xyz and modifies it as follows:
    !
    ! 1. In the cylinder between z = [base,top] with the radius around [center_x,center_y], the value
    !    
    !      dq*cos( pi/2 * (sqrt( (x(i)-center_x)**2 + (y(j)-center_y)**2 ) / radius) )**2
    !    
    !    is added locally to the water vapor.
    !
    ! 2. At each level, a horizontally constant value is added at each horizontal location so that the mean vertical
    !    water vapor profile is the same before and after applying the anomaly.
    
    implicit none
    
    !
    ! I/O
    !
    
    integer, intent(IN) :: x_n,y_n,z_n ! Grid dimension legnths
    
    double precision, intent(IN) :: dx,dy ! Horizontal grid spacing (m)
    
    double precision, intent(IN) :: z(z_n) ! Vertical grid (m)
    
    double precision, intent(INOUT) :: qv_xyz(x_n,y_n,z_n) ! 3D water vapor array (kg kg-1)
    
    ! Parameters of the mesoscale anomaly
    
    double precision, intent(IN) :: center_x ! Center of the anomaly in x-direction (m)
    double precision, intent(IN) :: center_y ! Center of the anomaly in y-direction (m)
    double precision, intent(IN) :: base     ! Base of the anomaly in z-direction (m)
    double precision, intent(IN) :: top      ! Top of the anomaly in z-direction (m)
    double precision, intent(IN) :: radius   ! Radius of the anomaly (m)
    double precision, intent(IN) :: dq       ! Full amplitude (trough to peak) of the anomaly (kg kg-1)
    
    !
    ! Local
    !
    
    integer :: i,j,k
    
    double precision :: rrr
    
    double precision,allocatable :: x(:),y(:) ! Locally generated horizontal grid dimensions
    
    double precision, allocatable :: mean0_qv_z(:),mean1_qv_z(:) ! Mean water vapor profiles
    
    double precision, parameter :: pi = 3.141592653589793D0
    
    !
    ! Do your worst
    !
    
    ! Construct the x-dimension
    
    if (allocated(x)) deallocate(x)
    allocate(x(x_n))
    
    do i = 1, x_n
      x(i) = dble(i)*dx
    enddo
    
    ! Construct the y-dimension
    
    if (allocated(y)) deallocate(y)
    allocate(y(y_n))
    
    do j = 1, y_n
      y(j) = dble(j)*dy
    enddo
    
    ! Allocate mean profile arrays
    
    if (allocated(mean0_qv_z)) deallocate(mean0_qv_z)
    allocate(mean0_qv_z(z_n))
    
    if (allocated(mean1_qv_z)) deallocate(mean1_qv_z)
    allocate(mean1_qv_z(z_n))
    
    ! Calculate domain mean profile of water vapor before applying the anomaly
    
    mean0_qv_z(:) = 0.0D0
    
    do k=1,z_n
      do j=1,y_n
        do i=1,x_n
          mean0_qv_z(k) = mean0_qv_z(k) + qv_xyz(i,j,k)
        end do
      end do
    end do
    
    mean0_qv_z(:) = mean0_qv_z(:)/dble(y_n*x_n)
    
    ! Apply the anomaly
    
    do k=1,z_n
      do j=1,y_n
        do i=1,x_n
          if((x(i)-center_x)**2.0D0+(y(j)-center_y)**2.0D0.lt.radius**2.0D0 &
            .and.(z(k).ge.base).and.(z(k).le.top)) then
             rrr = cos( pi/2.0D0 * (sqrt( (x(i)-center_x)**2.0D0 + (y(j)-center_y)**2.0D0 ) / radius) )**2.0D0
             qv_xyz(i,j,k) = qv_xyz(i,j,k) + dq*rrr
          end if
        end do
      end do
    end do
    
    ! Calculate domain mean profile of water vapor after applying the anomaly
    
    mean1_qv_z(:) = 0.0D0
    
    do k=1,z_n
      do j=1,y_n
        do i=1,x_n
          mean1_qv_z(k) = mean1_qv_z(k) + qv_xyz(i,j,k)
        end do
      end do
    end do
    
    mean1_qv_z(:) = mean1_qv_z(:)/dble(y_n*x_n)
    
    ! Adjust water vapor profile so that water vapor is conserved after applying the anomaly
    
    do k=1,z_n
      do j=1,y_n
        do i=1,x_n
          qv_xyz(i,j,k) = qv_xyz(i,j,k) - (mean1_qv_z(k) - mean0_qv_z(k))
        end do
      end do
    end do
    
  END SUBROUTINE initialize_water_vapor_mesoscale_anomaly
  
  ! ----------------------------------------------------------------------------
  
  SUBROUTINE initialize_water_vapor_z(z_n,z,qv_z)
    
    !
    ! Initializes the z dimension (m) (altitudes of levels) and the vertical profile of water vapor (kg kg-1)
    !
    
    implicit none
    
    !
    ! I/O
    !
    
    integer, intent(OUT) :: z_n
    
    double precision, allocatable, intent(OUT) :: z(:),qv_z(:)
    
    !
    ! Local
    !
    
    integer :: z_i
    
    !
    ! Do your worst
    !
    
    ! z-dimension
    
    z_n = 144
    
    if (allocated(z)) deallocate(z)
    allocate(z(z_n))
    
    z = (/ 10.0,40.0,80.0,120.0,160.0,200.0,240.0,280.0,320.0,360.0,400.0,440.0,480.0,520.0, &
      560.0,600.0,640.0,680.0,720.0,760.0,800.0,840.0,880.0,920.0,960.0,1000.0,1040.0, &
      1080.0,1120.0,1160.0,1200.0,1240.0,1280.0,1320.0,1360.0,1400.0,1440.0,1480.0, &
      1520.0,1560.0,1600.0,1640.0,1680.0,1720.0,1760.0,1800.0,1840.0,1880.0,1920.0,1960.0, &
      2000.0,2040.0,2080.0,2120.0,2160.0,2200.0,2240.0,2280.0,2320.0,2360.0,2400.0,2440.0, &
      2480.0,2520.0,2560.0,2600.0,2640.0,2680.0,2720.0,2760.0,2800.0,2840.0,2880.0,2920.0, &
      2960.0,3000.0,3040.0,3080.0,3120.0,3160.0,3200.0,3240.0,3280.0,3320.0,3360.0,3400.0, &
      3440.0,3480.0,3520.0,3560.0,3600.0,3640.0,3680.0,3720.0,3760.0,3800.0,3840.0,3880.0, &
      3920.0,3960.0,4000.0,4040.0,4080.0,4120.0,4160.0,4200.0,4240.0,4280.0,4320.0,4360.0, &
      4400.0,4440.0,4480.0,4520.0,4560.0,4600.0,4640.0,4680.0,4720.0,4760.0,4800.0,4840.0, &
      4880.0,4920.0,4960.0,5003.658203,5051.308594,5103.316406,5160.081055,5222.036133, &
      5289.657227,5363.462402,5444.016602,5531.937988,5627.899414,5732.636719,5846.951660, &
      5971.721191,6107.900879,6256.534180,6418.759766,6595.820801,6789.073730,7000.0 /)
    
    if (allocated(qv_z)) deallocate(qv_z)
    allocate(qv_z(z_n))
    
    ! Water vapor profile
    
    qv_z = (/14.993177,14.395863,14.243875,14.191271,14.152529,14.120543,14.093074,14.067711, &
      14.042986,14.017644,13.990619,13.959742,13.924470,13.871093,13.785021,13.597329, &
      13.236755,12.555718,11.627934,11.188296,11.035337,10.879358,10.672464,10.340525, &
      9.770117,9.011074,8.064609,6.912493,5.974755,5.473100,5.193038,5.060702,5.025218, &
      5.014993,5.001633,4.975550,4.933225,4.874476,4.796909,4.693653,4.563498,4.424000, &
      4.275169,4.116560,3.959883,3.819289,3.705854,3.615017,3.546377,3.506323,3.499897, &
      3.521293,3.553233,3.590769,3.630394,3.679354,3.732517,3.784715,3.811298,3.813979, &
      3.794820,3.736207,3.653119,3.546555,3.384829,3.158367,2.890034,2.575070,2.277572, &
      2.001085,1.753244,1.515954,1.299473,1.123129,0.988490,0.913278,0.856725,0.826849, &
      0.799609,0.778421,0.758610,0.740357,0.724406,0.709832,0.698505,0.688097,0.679077, &
      0.670181,0.661611,0.653092,0.645747,0.638637,0.634578,0.630752,0.630994,0.631366, &
      0.634595,0.637959,0.640965,0.643885,0.645687,0.647077,0.648049,0.648705,0.649061, &
      0.648901,0.648512,0.646585,0.644645,0.641053,0.637149,0.632416,0.626949,0.621073, &
      0.613432,0.605720,0.595868,0.585379,0.574505,0.562965,0.551403,0.539042,0.526580, &
      0.513750,0.500248,0.485637,0.473412,0.460683,0.454041,0.448874,0.448786,0.450691, &
      0.455661,0.469527,0.489608,0.506386,0.515441,0.517610,0.509004,0.473851,0.482820, &
      0.564347,0.595586,0.567321 /)
  
    qv_z = 0.001D0*qv_z ! g kg-1 -> kg kg-1
    
  END SUBROUTINE initialize_water_vapor_z
  
  ! ----------------------------------------------------------------------------
  
  SUBROUTINE initialize_water_vapor_xyz(x_n,y_n,z_n,dx,dy,x,y,z,qv_z,qv_xyz)
    
    !
    ! Initializes the x and y dimensions (m) and the 3D distribution of water vapor from its vertical profile (kg kg-1)
    !
    
    implicit none
    
    !
    ! I/O
    !
    
    integer, intent(IN) :: x_n,y_n,z_n
    
    double precision, intent(IN) :: dx,dy
    
    double precision, intent(IN) :: z(z_n),qv_z(z_n)
    
    double precision,allocatable, intent(OUT) :: x(:),y(:),qv_xyz(:,:,:)
    
    !
    ! Local
    !
    
    integer :: x_i,y_i,z_i
    
    !
    ! Do your worst
    !
    
    ! x-dimension
    
    if (allocated(x)) deallocate(x)
    allocate(x(x_n))
    
    do x_i = 1, x_n
      x(x_i) = dble(x_i)*dx
    enddo
    
    ! y-dimension
    
    if (allocated(y)) deallocate(y)
    allocate(y(y_n))
    
    do y_i = 1, y_n
      y(y_i) = dble(y_i)*dy
    enddo
    
    ! 3D distribution of water vapor
    
    if (allocated(qv_xyz)) deallocate(qv_xyz)
    allocate(qv_xyz(x_n,y_n,z_n))
    
    do z_i = 1, z_n
    do y_i = 1, y_n
    do x_i = 1, x_n
      qv_xyz(x_i,y_i,z_i) = qv_z(z_i)
    enddo
    enddo
    enddo
    
  END SUBROUTINE initialize_water_vapor_xyz
  
  ! ----------------------------------------------------------------------------
  
  SUBROUTINE save_data(unit,file_name,x_n,y_n,z_n,x,y,z,qv_xyz)
    
    implicit none
    
    integer, intent(IN) :: unit
    integer, intent(IN) :: x_n,y_n,z_n
    double precision, intent(IN) :: x(:),y(:),z(:)
    double precision, intent(IN) :: qv_xyz(:,:,:)
    
    character(len=*) :: file_name
    
    ! Open a binary file for writing
    open(unit=unit, file=file_name, form='unformatted', access='stream')
  
    ! Write the dimensions to the binary file
    write(unit) x_n, y_n, z_n
  
    ! Write the arrays to the binary file
    write(unit) x
    write(unit) y
    write(unit) z
    write(unit) qv_xyz
  
    ! Close the file
    close(unit)
    
  END SUBROUTINE save_data
    
END MODULE module_water_vapor_initialization
